open Ast

module Astprinter =
struct




    let rec string_of_stc (s: stc) =
        match s with
        | StcOfObjDef o -> "stc Node:" ^ string_of_obj_def o
        | StcOfFuncDef f -> "stc Node:" ^ string_of_func_def f
        | StcOfStmt s -> "stc Node:" ^ string_of_stmt s


   
    and string_of_func_def (f:func_def) =
        match f with
        | FuncDefOfFuncDecAndStcList (f, tp, lst) -> "func_def Node: \n" ^ " 1. " ^ string_of_func_dec f ^ "\n 2." ^ string_of_cltype tp ^ "\n 3." ^ string_of_in_func_stc_list lst 

    and string_of_func_dec (f:func_dec) = 
        match f with
        | FuncDeclaratorOfIDAndParameters (s,lst) -> "func_dec Node: \n" ^ " 1. " ^ s ^"\n 2. "^ string_of_parameter_list lst


    and string_of_in_func_stc (i:in_func_stc) = 
        match i with
        | InFuncStcOfStmt s -> "in_func_stc Node:" ^ string_of_stmt s


    and string_of_obj_def (o:obj_def) =
        match o with
        | ObjDefOfIDAndStcList (s, lst) -> "obj_def Node: \n" ^ "1." ^ s ^ "\n 2. " ^ string_of_in_obj_stc_list lst

    and string_of_in_obj_stc (i:in_obj_stc) =
        match i with
        | InObjStcOfVarDec v -> "in_obj_stc Node:" ^ string_of_var_dec v


    and string_of_cltype (c:cltype) =
        match c with
        | CLTypeOfInt -> "cltype Node: int"
        | CLTypeOfBoolean -> "cltype Node: Boolean"
        | CLTypeOfObjID s -> "cltype Node: " ^ s
        | CLTypeOfVoid -> "cltype Node: Void"
        | CLTypeOfArray (c, li) -> "cltype Node:" ^ string_of_cltype c
        | CLTypeOfUndeclare -> "cltype Node: Undeclare"
        | CLTypeOfReal -> "cltype Node: Real"


    and string_of_var_dec (v:var_dec) =
        match v with
        | VarDecOfTypeAndDeclaratorList (tp,lst) -> "var_dec Node: \n" ^ " 1. " ^ string_of_cltype tp ^ " \n 2. " ^ string_of_declarator_list lst

   

    and string_of_declarator (d:declarator) =
    match d with
    | DeclaratorOfID (s,i) -> "declarator Node: \n" ^ "1." ^ s ^ "\n 2." ^ string_of_int i
    | DeclaratorOfArray (s,i) -> "declarator Node: \n" ^ "1." ^ s ^ "\n 2." ^ string_of_int i
    | DeclaratorOfFixedArray (s,e,i) -> "declarator Node: \n" ^ "1." ^ s ^ "\n 2." ^ string_of_cst_expr e ^ "\n 3." ^ string_of_int i

    and string_of_stmt (s:stmt) = 
    match s with
    | StmtOfVarDec v -> "stmt Node: \n" ^ string_of_var_dec v
    | StmtOfExprStmt e -> "stmt Node: \n" ^ string_of_expr_stmt e
    | StmtOfSelectionStmt s -> "stmt Node: \n" ^ string_of_selection_stmt s
    | StmtOfIterationStmt i -> "stmt Node: \n" ^ string_of_iteration_stmt i
    | StmtOfJumpStmt j -> "stmt Node: \n" ^ string_of_jump_stmt j
    | StmtOfDrawStmt d -> "stmt Node: \n" ^ string_of_draw_stmt d

    and string_of_expr_stmt (es: expr_stmt) =
        match es with
        ExprStmtOfExpr es -> string_of_expr es

    and string_of_draw_stmt (d:draw_stmt) = 
    match d with
    | DrawStmtOfFD e -> "draw_stmt Node: \n" ^ string_of_expr e
    | DrawStmtOfBK e -> "draw_stmt Node: \n" ^ string_of_expr e
    | DrawStmtOfLF e -> "draw_stmt Node: \n" ^ string_of_expr e
    | DrawStmtOfRT e -> "draw_stmt Node: \n" ^ string_of_expr e
    | DrawStmtOfReset -> "draw_stmt Node: Reset"
    | DrawStmtOfCls -> "draw_stmt Node: Cls"
    | DrawStmtOfPf -> "draw_stmt Node: Pf"
    | DrawStmtOfPd -> "draw_stmt Node: Pd"
    | DrawStmtOfPu -> "draw_stmt Node: Pu"
    | DrawStmtOfWd e -> "draw_stmt Node: \n" ^ string_of_expr e
    | DrawStmtOfRGB (e1,e2,e3) -> "draw_stmt Node: \n" ^ " 1. " ^ string_of_expr e1 ^ " \n 2. " ^ string_of_expr e2 ^ " \n 3. " ^ string_of_expr e3

    and string_of_selection_stmt (s:selection_stmt) = 
    match s with
    | SelectionStmtOfExprAndStmtList (e,lst) -> "selection_stmt Node: \n" ^ " 1. " ^ string_of_expr e ^ "\n 2." ^ string_of_stmt_list lst
    | SelectionStmtOfExprAndStmtList2 (e,lst1,lst2) ->  "selection_stmt Node: \n" ^ " 1. " ^ string_of_expr e ^ "\n 2." ^ string_of_stmt_list lst1 ^ "\n 3." ^ string_of_stmt_list lst2

    and string_of_iteration_stmt (i:iteration_stmt) =
    match i with
    | IterationStmtOfExprAndStmtList (e,lst) -> "iteration_stmt Node: \n" ^ " 1. " ^ string_of_expr e ^ "\n 2." ^ string_of_stmt_list lst
    | IterationStmtOfStmtList lst -> "iteration_stmt Node: \n" ^  string_of_stmt_list lst

    and string_of_jump_stmt (j: jump_stmt) =
    match j with
    | JumpStmtOfGOON li -> "jump_stmt Node: GOON"
    | JumpStmtOfBRK li -> "jump_stmt Node: BRK"
    | JumpStmtOfReturnExpr e -> "jump_stmt Node: \n" ^ string_of_expr e
    | JumpStmtOfReturn li-> "jump_stmt Node: Return"


    and string_of_expr (e: expr) =
    match e with
    | ExprOfPrimary (p,i) -> "expr Node: \n" ^ " 1. " ^ string_of_primary p ^ " \n 2. " ^ string_of_int i
    | ExprOfCstExpr (c,i) -> "expr Node: \n" ^ " 1. " ^ string_of_cst_expr c ^ " \n 2." ^ string_of_int i
    | ExprOfAssign (l,e) -> "expr Node: \n" ^ " 1. " ^ string_of_lvalue l ^ " \n 2." ^ string_of_expr e 
    | ExprOfPlus (e1,e2) -> "expr Node: \n" ^ " 1. " ^ string_of_expr e1 ^ " \n 2." ^ string_of_expr e2
    | ExprOfMinus (e1,e2) -> "expr Node: \n" ^ " 1. " ^ string_of_expr e1 ^ " \n 2." ^ string_of_expr e2 
    | ExprOfTimes (e1,e2) -> "expr Node: \n" ^ " 1. " ^ string_of_expr e1 ^ " \n 2." ^ string_of_expr e2  
    | ExprOfDivide (e1,e2) -> "expr Node: \n" ^ " 1. " ^ string_of_expr e1 ^ " \n 2." ^ string_of_expr e2
    | ExprOfNeq (e1,e2) -> "expr Node: \n" ^ " 1. " ^ string_of_expr e1 ^ " \n 2." ^ string_of_expr e2  
    | ExprOfEq (e1,e2) -> "expr Node: \n" ^ " 1. " ^ string_of_expr e1 ^ " \n 2." ^ string_of_expr e2 
    | ExprOfLt (e1,e2) -> "expr Node: \n" ^ " 1. " ^ string_of_expr e1 ^ " \n 2." ^ string_of_expr e2  
    | ExprOfGt (e1,e2) -> "expr Node: \n" ^ " 1. " ^ string_of_expr e1 ^ " \n 2." ^ string_of_expr e2 
    | ExprOfLeq (e1,e2) -> "expr Node: \n" ^ " 1. " ^ string_of_expr e ^ " \n 2." ^ string_of_expr e2 
    | ExprOfGeq (e1,e2) -> "expr Node: \n" ^ " 1. " ^ string_of_expr e1 ^ " \n 2." ^ string_of_expr e2 
    | ExprOfUMinus e -> "expr Node: \n" ^ string_of_expr e  
    | ExprOfNot e -> "expr Node: \n" ^ string_of_expr e
    | ExprOfFunction (s,lst, li) -> "expr Node: \n" ^ " 1. " ^ s ^ " \n 2. " ^ string_of_expr_list lst


    and string_of_identifier (i:identifier) =
    match i with
    | IdentifierOfSimple (s,i) -> "identifier Node: \n" ^ " 1. " ^ s ^ " \n 2. " ^ string_of_int i 
    | IdentifierOfObjElement (s,i1,i2) -> "identifier Node: \n" ^ " 1. " ^ s ^ " \n 2. " ^ string_of_identifier i1 ^ " \n 3. " ^ string_of_int i2 

    and string_of_primary (p:primary) =
    match p with 
    | PrimaryOfIdentifier (i1,i2) -> "primary Node: \n" ^ " 1. " ^ string_of_identifier i1 ^ " \n 2. " ^ string_of_int i2
    | PrimaryOfArrayElement (i,e) -> "primary Node: \n" ^ " 1. " ^ string_of_identifier i ^ " \n 2. " ^ string_of_expr e

    and string_of_cst_expr (c:cst_expr) =
    match c with 
    | CstExprOfInt i -> "cst_expr Node: \n" ^ string_of_int i
    | CstExprOfReal f -> "cst_expr Node: \n" ^ string_of_float f
    | CstExprOfTrue -> "cst_expr Node: True"
    | CstExprOfFalse -> "cst_expr Node: False"

    and string_of_lvalue (l:lvalue) = 
    match l with
    | LvalueOfIdentifier (i1,i2) -> "lvalue Node: \n" ^ " 1. " ^ string_of_identifier i1 ^ " \n 2. " ^ string_of_int i2
    | LvalueOfArrayElement (i,e) -> "lvalue Node: \n" ^ " 1. " ^ string_of_identifier i ^ " \n 2. " ^ string_of_expr e
 

    and string_of_in_func_stc_list (lst: in_func_stc list) =
        match lst with
        | [] -> ""
        | head :: [] -> string_of_in_func_stc head 
        | head :: tail -> 
            let head = string_of_in_func_stc head in
            let tail = string_of_in_func_stc_list tail in
            head ^ "\n" ^ tail

    and string_of_parameter_list (lst: parameter list) =
        match lst with
        | [] -> ""
        | head :: [] -> string_of_parameter head 
        | head :: tail -> 
            let head = string_of_parameter head in
            let tail = string_of_parameter_list tail in
            head ^ "\n" ^ tail


    and string_of_parameter (p: parameter) = 
        match p with
        | ParameterOfTypeAndID (tp, id) -> "parameter Node: 1." ^ string_of_cltype  tp ^ "\n 2. " ^ id
        
    
    and string_of_in_obj_stc_list (lst: in_obj_stc list) =
        match lst with
        | [] -> ""
        | head :: [] -> string_of_in_obj_stc head 
        | head :: tail -> 
            let head = string_of_in_obj_stc head in
            let tail = string_of_in_obj_stc_list tail in
            head ^ "\n" ^ tail
        
    and string_of_declarator_list (lst: declarator list) =
        match lst with
        | [] -> ""
        | head :: [] -> string_of_declarator head 
        | head :: tail -> 
            let head = string_of_declarator head in
            let tail = string_of_declarator_list tail in
            head ^ "\n" ^ tail

    and string_of_stmt_list (lst: stmt list) =
        match lst with
        | [] -> ""
        | head :: [] -> string_of_stmt head 
        | head :: tail -> 
            let head = string_of_stmt head in
            let tail = string_of_stmt_list tail in
            head ^ "\n" ^ tail
    
    and string_of_expr_list (lst: expr list) =
        match lst with
        | [] -> ""
        | head :: [] -> string_of_expr head 
        | head :: tail -> 
            let head = string_of_expr head in
            let tail = string_of_expr_list tail in
            head ^ "\n" ^ tail


    and string_of_program (lst: stc list) = 
        match lst with
        [] -> "Null Program"
        | head::[] -> string_of_stc head ^ "\n"
        | head::tail -> 
            let h = string_of_stc head in
            let t = string_of_program tail in
            h ^ "\n" ^ t


    and print p =
        print_string (string_of_program p)
    
end
 
