open Header
open Astprinter
open Semantic
open Parser
open Generator


type action = Ast | Compile


let generate_code (program: Ast.program) =
    let str = Generator.generate program in
    (*print_string str;
    flush stdout;*)
    let lines = ref [] in
    (*read from template file*)
    let file = open_in "template.html" in
    for i = 1 to 48 do
        lines := input_line file :: !lines
    done;
    let s1 = String.concat "\n" (List.rev !lines) ^ "\n" in
    let s2 = 
        try 
            lines := [];
            while true do
                lines := input_line file :: !lines
            done;
            ""
        with End_of_file ->
            close_in file;
            String.concat "\n" (List.rev !lines) ^ "\n"
    in
    let file = open_out Sys.argv.(3) in
    (*concate template file and compiled js code*)
    Printf.fprintf file "%s" (s1 ^ str ^ s2);
    close_out file


let _ =
    let action = if Array.length Sys.argv == 3 then
        List.assoc Sys.argv.(1) 
            [ ("-a", Ast); 
            ("-c", Compile) ]
    else Compile in

    let srcfile = open_in Sys.argv.(2) in
    let lexbuf = Lexing.from_channel srcfile in

    try
        let program = Parser.program Scanner.token lexbuf in

        (*if Parser.parsing_succeed == true then*)

        match action with

            Ast -> Astprinter.print program

            | Compile -> 
                Semantic.check program;
                match !Semantic.checking_succeed with
                false -> ()
                | true -> 
                    generate_code program

    with IllegalCharacterError (ch, li, col) ->
        begin
            let errormsg = "Illegal token \'" ^ ch ^ "\' at line " ^ string_of_int li ^ ", column " ^ string_of_int col in
            Printf.printf "Syntax Error: ";
            Printf.printf "%s\n" errormsg;
        end

    | Parsing.Parse_error ->
        let line = linecounter lexbuf in
        Printf.printf "Syntax Error at line: %d\n" line


                

